include<lib.scad>

$fa = 1;
$fs = 0.2;

delta = 0.005;

height = 54.3;
width = 86.9;
thickness = 2.85;
height_secondary = 46.3;
width_secondary = 67.26;
thickness_secondary = 5.0;

cutout_width = 65.1;
cutout_radius = 2.5;

lower_ear_width = 8.1;
lower_ear_extension_distance = 5.85;
lower_ear_hole_offset = 1.9;
lower_ear_hole_diameter = 4.35;
lower_ear_thickness = 2.25;
lower_ear_inset_distance = 1.5;
lower_ears_distance = 30.2;

lower_ear_center_offset = lower_ear_extension_distance - lower_ear_hole_offset - (lower_ear_hole_diameter / 2);

inset_hole_diameter_inner = 4.3;
inset_hole_diameter_outer = 6.5;
inset_hole_inset = 2.65;
inset_hole_offset_double = 2.5;
inset_hole_offset_single = 19.9;
inset_holes_distance = 22.4;

inset_hole_offset_double_heightwise = height_secondary - inset_hole_diameter_outer/2 - inset_hole_offset_double;
inset_hole_offset_single_heightwise = height_secondary - inset_hole_diameter_outer/2 - inset_hole_offset_single;
inset_hole_offset_double_widthwise = (inset_holes_distance + inset_hole_diameter_outer) / 2;

board_mount_clearance_rectircle_width = 6.45;
board_mount_clearance_rectircle_inset = 7.0;
board_mount_clearance_rectircle_length = board_mount_clearance_rectircle_inset-board_mount_clearance_rectircle_width/2;
board_mount_clearance_rectircle_offset = 31.95;

module lower_ear_hole(diameter, offst) {
    translate([0,-offst+delta,0])
    circle(diameter/2);
}

module lower_ear_inset_hole(distance, offst) {
    translate([0,-offst+delta,0])
    circle(distance+offst);
}

module lower_ear(width, offst) {
    translate([0,-offst+delta,0])
    rectircle(width/2, offst);    
}

module inset_hole(diameter_inner, diameter_outer, inset, height) {
    translate([0,0,-delta])
    ihole(diameter_inner, diameter_outer, inset+delta, height+2*delta);
}

module vrm_heatsink() {
    difference() {
        union() {
            // Main Rectangle
            linear_extrude(thickness)
            translate([-width/2,0,0])
            square([width, height]);
            
            // Secondary Rectangle (Thicker Portion)
            linear_extrude(thickness_secondary)
            translate([-width_secondary/2,0,0])
            square([width_secondary, height_secondary]);
            
            // Lower Ear (Right)
            linear_extrude(lower_ear_thickness)
            translate([(lower_ears_distance+lower_ear_width)/2,0,0])
            lower_ear(lower_ear_width, lower_ear_center_offset);
            
            // Lower Ear (Left)
            linear_extrude(lower_ear_thickness)
            translate([-(lower_ears_distance+lower_ear_width)/2,0,0])
            lower_ear(lower_ear_width, lower_ear_center_offset);
        }
        
        // Top Cutout
        translate([-cutout_width/2,height_secondary+delta,-delta])
        difference() {
            linear_extrude(thickness+2*delta)
            square([cutout_width,height-height_secondary+delta]);
            
            rounding(delta, cutout_radius, thickness+2*delta, 180);
            
            translate([cutout_width,0,0])
            rounding(delta, cutout_radius, thickness+2*delta, -90);
        }
            
        // Lower Ear Hole (Right)
        translate([0,0,-delta])
        linear_extrude(thickness_secondary+2*delta)
        translate([(lower_ears_distance+lower_ear_width)/2,0,0])
        lower_ear_hole(lower_ear_hole_diameter, lower_ear_center_offset);
        
        // Lower Ear Hole (Left)
        translate([0,0,-delta])
        linear_extrude(thickness_secondary+2*delta)
        translate([-(lower_ears_distance+lower_ear_width)/2,0,0])
        lower_ear_hole(lower_ear_hole_diameter, lower_ear_center_offset);
        
        // Lower Ear Secondary Rectangle Cutout (Right)
        translate([0,0,lower_ear_thickness])
        linear_extrude(thickness_secondary-lower_ear_thickness+delta)
        translate([(lower_ears_distance+lower_ear_width)/2,0,0])
        lower_ear_inset_hole(lower_ear_inset_distance, lower_ear_center_offset);
        
        // Lower Ear Secondary Rectangle Cutout (Left)
        translate([0,0,lower_ear_thickness])
        linear_extrude(thickness_secondary-lower_ear_thickness+delta)
        translate([-(lower_ears_distance+lower_ear_width)/2,0,0])
        lower_ear_inset_hole(lower_ear_inset_distance, lower_ear_center_offset);
        
        // Inset Hole (Center)
        translate([0,inset_hole_offset_single_heightwise,0])
        inset_hole(inset_hole_diameter_inner, inset_hole_diameter_outer, inset_hole_inset, thickness_secondary);
        
        // Inset Hole (Right)
        translate([inset_hole_offset_double_widthwise,inset_hole_offset_double_heightwise,0])
        inset_hole(inset_hole_diameter_inner, inset_hole_diameter_outer, inset_hole_inset, thickness_secondary);
        
        // Inset Hole (Left)
        translate([-inset_hole_offset_double_widthwise,inset_hole_offset_double_heightwise,0])
        inset_hole(inset_hole_diameter_inner, inset_hole_diameter_outer, inset_hole_inset, thickness_secondary);
        
        // Board Mount Clearance Coutout (Right)
        translate([width/2-board_mount_clearance_rectircle_length+delta,board_mount_clearance_rectircle_offset+board_mount_clearance_rectircle_width/2,-delta])
        rotate([0,0,-90])
        linear_extrude(thickness+2*delta)
        rectircle(board_mount_clearance_rectircle_width/2, board_mount_clearance_rectircle_length);
        
        // Board Mount Clearance Coutout (Left)
        translate([-width/2+board_mount_clearance_rectircle_length-delta,board_mount_clearance_rectircle_offset+board_mount_clearance_rectircle_width/2,-delta])
        rotate([0,0,90])
        linear_extrude(thickness+2*delta)
        rectircle(board_mount_clearance_rectircle_width/2, board_mount_clearance_rectircle_length);
    }
}

vrm_heatsink();